//
//  DataManager.m
//  stylepop
//
//  Created by 경영 임 on 13. 1. 22..
//  Copyright (c) 2013년 Pinatalabs. All rights reserved.
//

#import "DataManager.h"
#import "SVHTTPRequest.h"
#import "SplashViewController.h"

@implementation DataManager

__strong static id _sharedObject = nil;

+ (id)sharedInstance
{
    if( !_sharedObject )
            _sharedObject = [[self alloc] init];
    return _sharedObject;
}

-(id) init
{
    if( self ) {
        [super init];
        self.arrayEventDic = nil;
    }
    return self;
}

-(void) dealloc
{
    [_arrayEventDic removeAllObjects];
    _arrayEventDic = nil;
    [super dealloc];
}

-(void) sendVersionCheck
{
    [SVHTTPRequest POST:[NSString stringWithFormat:@"%@/version_check.json",_address]
             parameters:[NSDictionary dictionaryWithObjectsAndKeys:
                         [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleVersion"], @"client_ver", nil]
             completion:^(id response,NSHTTPURLResponse *urlResponse, NSError *error)
     {
         if( urlResponse.statusCode != 200 ) return;
         if( error )
         {
             NSLog(@"FetchAuthToken Error %@",[error description]);
             return ;
         }
     }];
}

#pragma mark - Global Error Handler
-(void)globalErrorHandler
{
    [SVHTTPRequest setsGlobalErrHandler:^(id response, NSHTTPURLResponse *urlResponse, NSError *error)
     {
//        NSLog(@"Global Handler Response %@ urlResponseCode %d error : %@",response,urlResponse.statusCode,error);
        if( urlResponse.statusCode==500 || urlResponse.statusCode == 502 || urlResponse.statusCode == 503)
        {
            UIAlertView* alert = [[UIAlertView alloc] initWithTitle:@"경고" message:@"서버와의 연결이 원할하지 않습니다" delegate:nil cancelButtonTitle:@"확인" otherButtonTitles:nil];
            [alert show];
        }
        else if( urlResponse.statusCode == 401)  //authorization error
        {
            [[SignManager sharedInstance] setAuth_token:nil];
            [SignManager saveToUserDefaults:nil forKey:@"auth_token"];
            UIAlertView* alert = [[UIAlertView alloc] initWithTitle:@"오류" message:@"사용자의 유저정보가 정확하지 않습니다 다시 로그인해주세요." delegate:nil cancelButtonTitle:@"확인" otherButtonTitles: nil];
            [alert show];
        }
        else if( urlResponse.statusCode==444 )
        {
            UIAlertView* alert = [[UIAlertView alloc] initWithTitle:@"경고" message:[NSString stringWithFormat:@"%@ (requre ver. %@)",[response objectForKey:@"error"],[response objectForKey:@"client_ver"]] delegate:nil cancelButtonTitle:@"확인" otherButtonTitles: nil];
            [alert show];
        }
        else if( urlResponse.statusCode>=300 )
        {
            if( [response isKindOfClass:[NSDictionary class]])
            {
                if( [response objectForKey:@"error"] )
                {
                    UIAlertView* alert = [[UIAlertView alloc] initWithTitle:@"경고" message:[NSString stringWithFormat:@"%@ (requre ver. %@)",[response objectForKey:@"error"],[response objectForKey:@"client_ver"]] delegate:nil cancelButtonTitle:@"확인" otherButtonTitles: nil];
                    [alert show];
                    return;
                }
            }
            UIAlertView* alert = [[UIAlertView alloc] initWithTitle:@"경고" message:@"오류가 발생하였습니다" delegate:nil cancelButtonTitle:@"확인" otherButtonTitles: nil];
            [alert show];
        }
        else if( urlResponse.statusCode == 0)
        {
            if(error)
            {
               UIAlertView* alert =  [[UIAlertView alloc] initWithTitle:@"오류" message:[error localizedDescription] delegate:nil cancelButtonTitle:@"확인" otherButtonTitles: nil];
               [alert show];
            }
        }
     }
    ];
}

-(void) setEventListArray:(NSArray*)array
{
    self.arrayEventDic = [[NSMutableArray alloc] initWithArray:array];
}

+(void) fetch_EventList:(void (^)(BOOL isSuccess)) endBlock
{
//    auth_token 오류가능
    [SVHTTPRequest GET:[NSString stringWithFormat:@"%@/eventlist.json",_address]
            parameters:[NSDictionary dictionaryWithObjectsAndKeys:[[SignManager sharedInstance] auth_token],@"auth_token",nil]
            completion:^(id response, NSHTTPURLResponse *urlResponse, NSError *error) {
                if( !error && urlResponse.statusCode==200 )
                {
                    NSLog(@"%@", [response objectForKey:@"entries"]);
                    [[DataManager sharedInstance] setEventListArray:[response objectForKey:@"entries"]];
                    endBlock(true);
                    return;
                }
                endBlock(false);
            }
     ];
}


@end
