//
//  SignManager.m
//  stylepop
//
//  Created by 경영 임 on 13. 1. 23..
//  Copyright (c) 2013년 Pinatalabs. All rights reserved.
//

#import "SignManager.h"
#import <FacebookSDK/FacebookSDK.h>
#import "AppDelegate.h"
#import "SplashViewController.h"
#import "MainViewController.h"
#import "EventListViewController.h"
#import "SVHTTPRequest.h"

@implementation SignManager


__strong static id _sharedObject = nil;

+ (id)sharedInstance
{
    if( !_sharedObject )
        _sharedObject = [[self alloc] init];
    return _sharedObject;
}


- (id)init
{
    self = [super init];
    if (self) {
        self.auth_token = nil;
        [self startObservingKeys];
    }
    return self;
}

- (void)dealloc
{
    [_device_token release];
    [_auth_token release];
    [_pUserOwnInfo release];
    [super dealloc];
}


- (void)startObservingKeys
{
    [self addObserver:self
           forKeyPath:@"device_token"
              options:NSKeyValueObservingOptionNew|NSKeyValueObservingOptionOld
              context:nil];
    [self addObserver:self
           forKeyPath:@"auth_token"
              options:NSKeyValueObservingOptionNew|NSKeyValueObservingOptionOld
              context:nil];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    if (object == self)
    {   
        NSLog(@"keypath %@ change %@",keyPath,change);
        if(self.device_token && self.auth_token)
        {
            if(self.needSendDeviceToken)
            {
                //                [self registerPushDevice:^(BOOL isSuccess, NSString *result) {
                //                }];
                self.needSendDeviceToken = NO;
            }
        }
    }
    else
    {
        [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
    }
}


+(id) loadFromUserDefaults:(id)key
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    id returnVal = nil;
    if(userDefaults && key)
    {
        returnVal = [userDefaults objectForKey:key];
    }
    return returnVal;
}

+(BOOL) saveToUserDefaults:(id)object forKey:(id)key
{
    BOOL returnVal = NO;
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    @synchronized(userDefaults) {
        if(userDefaults && key && object)
        {
            [userDefaults setObject:object forKey:key];
        }
        else
        {
            [userDefaults removeObjectForKey:key];
        }
        returnVal = [userDefaults synchronize];
    }
    return returnVal;
}


-(void)loadLoginInfo
{
    self.auth_token = (NSString *)[SignManager loadFromUserDefaults:@"auth_token"];
    self.device_token = (NSString *)[SignManager loadFromUserDefaults:@"device_token"];
    self.userOwnID = [[SignManager loadFromUserDefaults:@"current_user_id"] integerValue];
    self.isSetPassword = [[SignManager loadFromUserDefaults:@"set_password"] boolValue];
    self.isFBLogin = [[SignManager loadFromUserDefaults:@"isfblogin"] boolValue];
}

-(void) saveLoginInfo
{
    [SignManager saveToUserDefaults:[NSNumber numberWithBool:self.isFBLogin] forKey:@"isfblogin"];
    [SignManager saveToUserDefaults:self.auth_token forKey:@"auth_token"];
    [SignManager saveToUserDefaults:[NSNumber numberWithInteger:self.userOwnID]  forKey:@"current_user_id"];
    [SignManager saveToUserDefaults:[NSNumber numberWithBool:self.isSetPassword] forKey:@"set_password"];
    [SignManager saveToUserDefaults:self.device_token forKey:@"device_token"];
}

-(void) actionAfterLogin: (NSDictionary *)response
{
    self.isFBLogin = NO;
    self.auth_token = [response valueForKey:@"auth_token"];
    self.userOwnID = [[response objectForKey:@"current_user_id"] integerValue];
    self.isSetPassword = [[response objectForKey:@"set_password"] boolValue];
    [self saveLoginInfo];
    [SignManager SignInSuccess];
}

-(void) actionAfterFBLogin: (NSDictionary *)response
{
    self.isFBLogin = YES;
    self.auth_token = [response valueForKey:@"auth_token"];
    self.userOwnID = [[response objectForKey:@"current_user_id"] integerValue];
    self.isSetPassword = [[response objectForKey:@"set_password"] boolValue];
    [self saveLoginInfo];
}

-(void) actionAfterSignOut
{
    self.isFBLogin = NO;
    self.auth_token = nil;
    self.userOwnID = 0;
    self.isSetPassword = NO;
    [self saveLoginInfo];
    [FBSession.activeSession closeAndClearTokenInformation];
    AppDelegate *appdelegate = (AppDelegate *)[UIApplication sharedApplication].delegate;
    SplashViewController * splVC= [[SplashViewController alloc] init];
    [appdelegate.navigationController setViewControllers:[NSArray arrayWithObjects:splVC,nil]];
}

+(void) send_SignIn:(NSDictionary *)dic endBlock:(void (^)(BOOL isSuccess)) endBlock
{
    [SVHTTPRequest POST:[NSString stringWithFormat:@"%@/users/sign_in.json",_address] parameters:dic completion:^(id response,NSHTTPURLResponse *urlResponse, NSError *error)
     {
         NSLog(@"response : %@ status:%d",[response description],urlResponse.statusCode);
         if(error || urlResponse.statusCode !=200)
         {
             NSLog(@"FetchAuthToken Error %@",[error description]);
             endBlock(false);
             return ;
         }
         [[SignManager sharedInstance] actionAfterLogin:response];
         endBlock(true);
     }];
}

+(void) send_SignOut:(void (^)(BOOL isSuccess)) endBlockwithResult
{
    SignManager *sMgr = [SignManager sharedInstance];
    [SVHTTPRequest GET:[NSString stringWithFormat:@"%@/signout.json",_address] parameters:[NSDictionary dictionaryWithObjectsAndKeys:sMgr.auth_token,@"auth_token", nil] completion:^(id response,NSHTTPURLResponse *urlResponse, NSError *error)
     {
         if(urlResponse.statusCode !=200 || error)
         {
             NSLog(@"SignOut Error %@",[error description]);
             endBlockwithResult(false);
             return ;
         }
         NSLog(@"response : %@",[response description]);
         [[SignManager sharedInstance] actionAfterSignOut];
         endBlockwithResult(true);
     }];
}

+(void) send_SignUp:(NSDictionary *)dic endBlockwithResult:(void (^)(BOOL isSuccess)) endBlockwithResult
{
    [SVHTTPRequest POST:[NSString stringWithFormat:@"%@/users.json",_address]
             parameters:dic
             completion:^(id response,NSHTTPURLResponse *urlResponse, NSError *error)
     {
         if(urlResponse.statusCode !=201|| error)
         {
             NSLog(@"FetchAuthToken Error %@",[error description]);
             endBlockwithResult(false);
             return ;
         }
         [[SignManager sharedInstance] actionAfterLogin:response];
         endBlockwithResult(true);
     }];
}

+(void) send_MFB_SIGNIN:(void (^)(BOOL isSuccess)) endBlock
{
    if(![FBSession activeSession].accessToken || ![FBSession activeSession].isOpen)
    {
        endBlock(false);
        return;
    }
    [SVHTTPRequest POST:[NSString stringWithFormat:@"%@/mfb_signin.json",_address] parameters:[NSDictionary dictionaryWithObject:[FBSession activeSession].accessToken forKey:@"ftoken"] completion:^(id response,NSHTTPURLResponse *urlResponse, NSError *error){
        if(urlResponse.statusCode !=200 || error)
        {
            [FBSession.activeSession closeAndClearTokenInformation];
            endBlock(false);
            return ;
        }
        [[SignManager sharedInstance] actionAfterFBLogin:response];
//        [[SignManager sharedInstance] loadLoginInfo];
        [SignManager SignInSuccess];
        endBlock(true);
    }];
}

+(void) SignInSuccess
{
    AppDelegate *appdelegate = (AppDelegate *)[UIApplication sharedApplication].delegate;
    
    MainViewController *nextVC = [[MainViewController alloc] init];
    [appdelegate.navigationController setViewControllers:[NSArray arrayWithObjects:nextVC,nil]];
    
}

@end
